package com.example.flutter_icon_network

import android.os.Build
import android.os.StrictMode
import android.util.Log
import androidx.annotation.NonNull
import foundation.icon.icx.data.Address
import foundation.icon.icx.data.Bytes
import foundation.icon.icx.data.TransactionResult
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import java.math.BigInteger
import java.nio.file.Path


class FlutterIconNetworkPlugin : FlutterPlugin, MethodChannel.MethodCallHandler {
    private lateinit var channel: MethodChannel
    private var networkId: String? = null
    private var host: String? = null

    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flutter_icon_network")
        channel.setMethodCallHandler(this)

        val SDK_INT = Build.VERSION.SDK_INT
        if (SDK_INT > 8) {
            val policy = StrictMode.ThreadPolicy.Builder()
                .permitAll().build()
            StrictMode.setThreadPolicy(policy)
        }
    }

    override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
        when (call.method) {
            "init" -> {
                networkId = call.argument("network_id")
                host = call.argument("host")
                result.success("{\"host\":\"${host}\",\"networkId\":\"${networkId}\\}")
            }
            "createWallet" -> {
                try {
                    val wallet: Map<String, String> = ICONWalletManager.getInstance().createWallet()
                    result.success("{\"private_key\":\"${wallet["private_key"]}\",\"address\":\"${wallet["address"]}\"}")

                } catch (e: Exception) {
                    result.error("CREATE_WALLET_ERROR_CODE", e.message, null)
                }
            }
            "sendIcx" -> {
                val from: String? = call.argument("from")
                val to: String? = call.argument("to")
                val value: String? = call.argument("value")
                Log.v("sendIcx: ", "from $from to $to value $value")
                try {
                    val txHash =
                        ICONTransactionManager.getInstance(host, networkId).sendICX(from, value, to)
                    result.success("{\"txHash\":\"${txHash}\",\"status\":0}")

                } catch (e: Exception) {
                    result.error("ICX_TRANSFER_ERROR_CODE", e.message, null)
                }
            }
            "getIcxBalance" -> {
                val privateKey: String? = call.argument("private_key")
                try {
                    val balance: BigInteger? =
                        ICONTransactionManager.getInstance(host, networkId)
                            .getICXBalance(privateKey)
                    result.success("{\"balance\":\"${balance}\"}")
                } catch (e: Exception) {
                    result.error("GET_ICX_BALANCE_ERROR_CODE", e.message, null)
                }
            }
            "sendToken" -> {
                val from: String? = call.argument("from")
                val to: String? = call.argument("to")
                val value: String? = call.argument("value")
                val scoreAddress: String? = call.argument("score_address")
                Log.v("sendIcx: ", "from $from to $to value $value")


                try {
                    val txHash = ICONSCOREManager.getInstance(host, networkId)
                        .sendToken(from, to, scoreAddress, value)
                    result.success("{\"txHash\":\"${txHash}\",\"status\":0}")
                } catch (e: Exception) {
                    result.error("SEND_TOKEN_ERROR_CODE", e.message, null)
                }
            }
            "getTokenBalance" -> {
                val privateKey: String? = call.argument("private_key")
                val scoreAddress: String? = call.argument("score_address")
                try {
                    val balance: BigInteger? = ICONSCOREManager.getInstance(host, networkId)
                        .getTokenBalance(privateKey, scoreAddress)
                    result.success("{\"balance\":\"${balance}\"}")
                } catch (e: Exception) {
                    result.error("GET_TOKEN_BALANCE_ERROR_CODE", e.message, null)
                }
            }
            "deployScore" -> {
                val privateKey: String? = call.argument("private_key")
                val initIcxSupply: String? = call.argument("init_icx_supply")

                try {
                    val tResult: TransactionResult = ICONSCOREManager.getInstance(host, networkId)
                        .deployScore(privateKey, initIcxSupply, call.argument("content"))
                    result.success("{\"score_address\":\"${tResult.scoreAddress}\", \"status\":\"${tResult.status}\",\"tx_hash\":\"${tResult.txHash}\", \"step_used\":\"${tResult.stepUsed}\"}")

                } catch (e: Exception) {
                    result.error("DEPLOY_SCORE_ERROR_CODE", e.message, null)
                }
            }
            "getLastBlock" -> {
                try {
                    val lastBlock: String? =
                        ICONSyncBlockManager.getInstance(host).lastBlock
                    result.success("{\"lastBlock\":\"${lastBlock}\"}")
                } catch (e: Exception) {
                    result.error("GET_LAST_BLOCK_ERROR_CODE", e.message, null)
                }
            }
            "getConfirmedTransactionList" -> {
                try {
                    val transactionIds: List<String>? =
                        ICONSyncBlockManager.getInstance(host).confirmedTransactionList
                    result.success(transactionIds)

                } catch (e: Exception) {
                    result.error("GET_CONFIRMED_TRANSACTION_LIST_ERROR_CODE", e.message, null)
                }
            }
            "getScoreTokenName" -> {
                val scoreAddress: String? = call.argument("score_address")
                try {
                    val tokenName: String? =
                        ICONSCOREManager.getInstance(host, networkId).checkTokenName(scoreAddress)
                    result.success("{\"tokenName\":\"${tokenName}\"}")
                } catch (e: Exception) {
                    result.error("GET_TOKEN_NAME_ERROR_CODE", e.message, null)
                }
            }
            "getTokenSymbol" -> {
                val scoreAddress: String? = call.argument("score_address")
                try {
                    val symbolToken: String? =
                        ICONSCOREManager.getInstance(host, networkId)
                            .checkSymbolToken(scoreAddress)
                    result.success("{\"tokenSymbol\":\"${symbolToken}\"}")
                } catch (e: Exception) {
                    result.error("GET_TOKEN_SYMBOL_ERROR_CODE", e.message, null)
                }
            }
            "getTransactionResult" -> {
                try {
                    val txHash: String? = call.argument("tx_hash")
                    val tResult =
                        ICONSCOREManager.getInstance(host, networkId)
                            .getTransactionResult(Bytes(txHash))
                    val data = mapOf<String, String>(
                        "block_hash" to tResult.blockHash.toString(),
                        "block_height" to tResult.blockHeight.toString(),
                        "cumulative_step_used" to tResult.cumulativeStepUsed.toString(),
                        "to" to tResult.to.toString(),
                        "txHash" to tResult.txHash.toString(),
                        "score_address" to tResult.scoreAddress,
                        "step_used" to tResult.stepUsed.toString(),
                        "status" to tResult.status.toString(),
                        "logs_bloom" to tResult.logsBloom,
                        "step_price" to tResult.stepPrice.toString(),
                        "tx_index" to tResult.txIndex.toString()
                    )

                    result.success(data)
                } catch (e: Exception) {
                    result.error("GET_TRANSACTION_RESULT_ERROR_CODE", e.message, null)
                }
            }
            "getBlockByHeight" -> {
                val blockHeight: String? = call.argument("block_height")
                try {
                    val block = ICONSyncBlockManager.getInstance(host)
                        .getBlockByHeight(BigInteger(blockHeight.toString()))

                    val data = mapOf<String, String>(
                        "block_hash" to block.blockHash.toString(),
                        "block_height" to block.height.toString(),
                        "merkle_tree_root_hash" to block.merkleTreeRootHash.toString(),
                        "peer_id" to block.peerId,
                        "prev_block_hash" to block.prevBlockHash.toString(),
                        "signature" to block.signature,
                        "time_stamp" to block.timestamp.toString()
                    )

                    result.success(data)
                } catch (e: Exception) {
                    result.error("GET_BLOCK_BY_HEIGHT_ERROR_CODE", e.message, null)
                }
            }
            "getBlockByHash" -> {
                val blockHash: String? = call.argument("block_hash")

                try {
                    val block =
                        ICONSyncBlockManager.getInstance(host).getBlockByHash(Bytes(blockHash))
                    val data = mapOf<String, String>(
                        "block_hash" to block.blockHash.toString(),
                        "block_height" to block.height.toString(),
                        "merkle_tree_root_hash" to block.merkleTreeRootHash.toString(),
                        "peer_id" to block.peerId,
                        "prev_block_hash" to block.prevBlockHash.toString(),
                        "signature" to block.signature,
                        "time_stamp" to block.timestamp.toString()
                    )
                    result.success(data)
                } catch (e: Exception) {
                    result.error("GET_BLOCK_BY_HASH_ERROR_CODE", e.message, null)
                }
            }
            "getTotalSupply" -> {
                try {
                    val totalSupply = ICONSyncBlockManager.getInstance(host).blockTotalSupply
                    result.success("{\"totalSupply\":\"${totalSupply}\"}")
                } catch (e: Exception) {
                    result.error("GET_TOTAL_SUPPLY_ERROR_CODE", e.message, null)
                }
            }
            "getScoreAPIs" -> {
                val scoreAddress: String? = call.argument("score_address")

                try {
                    val scoreAPIs =
                        ICONSCOREManager.getInstance(host, networkId)
                            .getBlockScoreAPI(Address(scoreAddress))
                    val data = mutableListOf<Map<String, String>>()
                    for (i in 0 until scoreAPIs.size - 1) {
                        data.add(
                            i,
                            mapOf(
                                "name" to scoreAPIs[i].name,
                                "read_only" to scoreAPIs[i].readonly,
                                "type" to scoreAPIs[i].type
                            )
                        )
                    }

                    result.success(data)
                } catch (e: Exception) {
                    result.error("GET_SCORE_APIS_ERROR_CODE", e.message, null)
                }
            }
            "getWalletByPrivateKey" -> {
                val privateKey: String? = call.argument("private_key")

                try {
                    val wallet = ICONWalletManager.getInstance().getWalletByPrivateKey(privateKey)

                    result.success(wallet)
                } catch (e: Exception) {
                    result.error("GET_WALLET_BY_PRIVATE_KEY_ERROR_CODE", e.message, null)
                }
            }
        }
    }

    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }
}
