import 'dart:async';
import 'dart:io';
import 'dart:typed_data';

import 'package:file_picker/file_picker.dart';
import 'package:flutter/services.dart';
import 'package:flutter_icon_network/models/balance.dart';
import 'package:flutter_icon_network/models/send_icx_response.dart';
import 'package:flutter_icon_network/models/transaction_result.dart';
import 'package:flutter_icon_network/models/wallet.dart';

export 'package:flutter_icon_network/constant.dart';
export 'package:flutter_icon_network/models/balance.dart';
export 'package:flutter_icon_network/models/send_icx_response.dart';
export 'package:flutter_icon_network/models/wallet.dart';

/// A Flutter plugin to work with native Android/Ios sdk of Icon network
/// Native sdk: https://www.icondev.io/docs/sdk-overview
class FlutterIconNetwork {
  static const MethodChannel _channel =
      const MethodChannel('flutter_icon_network');
  late String host;

  /// is test network or main network
  bool? isTestNet;

  /// run once in your main.dart file
  ///
  /// FlutterIconNetwork.instance.init(host: "https://bicon.net.solidwallet.io/api/v3", isTestNet: true);
  Future<void> init({required String host, bool? isTestNet}) async {
    this.isTestNet = isTestNet;
    this.host = host;

    await _initHostAndNetWorkId();
  }

  /// Init host and network id
  ///
  /// Init the global host and network id for the first time and then just use
  /// them and we don't need to pass them in every function
  Future<void> _initHostAndNetWorkId() async {
    final response = await _channel.invokeMethod("init", {
      "host": host,
      "network_id": _networkId,
    });

    if (response != null) {
      print(response);
    }
  }

  /// create a new wallet
  /// return the `Wallet` object that contain the `privateKey` and `address`
  ///
  /// ```final wallet = await FlutterIconNetwork.instance.createWallet;```
  Future<Wallet> get createWallet async {
    final String wallet = await (_channel.invokeMethod('createWallet'));
    print("FlutterIconNetwork createWallet $wallet");
    return walletFromJson(wallet);
  }

  /// return current icx balance
  ///
  /// ```final balance = await FlutterIconNetwork.instance.getIcxBalance(privateKey: yourPrivateKey);```
  Future<Balance> getIcxBalance({String? privateKey}) async {
    final String balance = await (_channel.invokeMethod('getIcxBalance', {
      'private_key': privateKey,
    }));
    print("FlutterIconNetwork getIcxBalance $balance");
    return balanceFromJson(balance);
  }

  /// value is decimal, ex: '1'
  /// send Icx to an `address`
  /// return the `transaction hash`
  /// ```
  /// final txHash = await FlutterIconNetwork.instance.sendIcx(
  ///                         yourPrivateKey: yourPrivateKey,
  ///                         destinationAddress: address,
  ///                         value: 1
  ///                      )
  /// ```
  Future<SendIcxResponse> sendIcx(
      {String? yourPrivateKey,
      String? destinationAddress,
      String? value}) async {
    final String response = await (_channel.invokeMethod('sendIcx', {
      "from": yourPrivateKey,
      "to": destinationAddress,
      "value": value,
    }));
    print("FlutterIconNetwork sendIcx $response");
    return sendIcxResponseFromJson(response);
  }

  /// to check the token balance in SCORE
  /// ```
  /// final balance = await FlutterIconNetwork.instance.getTokenBalance(
  ///                                     privateKey: privateKey,
  ///                                     scoreAddress: scoreAddress);
  /// ```
  Future<Balance> getTokenBalance(
      {String? privateKey, String? scoreAddress}) async {
    final String balance = await (_channel.invokeMethod('getTokenBalance', {
      'private_key': privateKey,
      'score_address': scoreAddress,
    }));
    print("FlutterIconNetwork getTokenBalance $balance");
    return balanceFromJson(balance);
  }

  ///deployContract: only available in android, on ios the sdk not support yet
  /// to deploy a SCORE by choose the zip file contain the source code
  /// the sample token zip file contain in source code, pls download it to your phone
  /// ```
  /// final transactionResult = await FlutterIconNetwork.instance.deployScore(
  ///                             privateKey: privateKey, initIcxSupply: "10");
  /// ```
  Future<TransactionResult?> deployScore(
      {String? privateKey, String? initIcxSupply}) async {
    if (Platform.isIOS) {
      return null;
    }
    Uint8List? fileBytes = await readFile();

    final String result = await (_channel.invokeMethod('deployScore', {
      'private_key': privateKey,
      'init_icx_supply': initIcxSupply,
      'content': fileBytes
    }));
    print("FlutterIconNetwork deployScore $result");
    return transactionResultFromJson(result);
  }

  Future<Uint8List?> readFile() async {
    FilePickerResult? result = await FilePicker.platform.pickFiles();
    if (result != null) {
      File file = File(result.files.single.path!);
      return await file.readAsBytes();
    } else {
      return null;
    }
  }

  /// value is num of icx, ex: 1
  /// to send token to a address through SCORE
  /// ```
  /// final response = await FlutterIconNetwork.instance.sendToken(
  ///         yourPrivateKey: privateKey,
  ///         toAddress: receiverAddress,
  ///         value: numOfToken,
  ///         scoreAddress: scoreAddress
  ///        );
  /// ```
  Future<SendIcxResponse> sendToken(
      {String? yourPrivateKey,
      String? toAddress,
      String? scoreAddress,
      String? value}) async {
    final String response = await (_channel.invokeMethod('sendToken', {
      "from": yourPrivateKey,
      "to": toAddress,
      "score_address": scoreAddress,
      "value": value,
    }));
    print("FlutterIconNetwork sendToken $response");
    return sendIcxResponseFromJson(response);
  }

  /// singleton
  static FlutterIconNetwork? get instance {
    _instance ??= FlutterIconNetwork._();
    return _instance;
  }

  static FlutterIconNetwork? _instance;

  FlutterIconNetwork._();

  String get _networkId {
    if (Platform.isAndroid) {
      if (isTestNet!) {
        return '3';
      } else {
        return '1';
      }
    } else {
      if (isTestNet!) {
        return '0x3';
      } else {
        return '0x1';
      }
    }
  }

  /// Get last Block
  Future<String?> getLastBlock() async {
    final String? response = await _channel.invokeMethod("getLastBlock");
    return response;
  }

  /// Get transaction id list
  Future<List<String>?> getConfirmedTransactionIdList() async {
    final List? response =
        await _channel.invokeMethod("getConfirmedTransactionList");
    if (response == null) return null;
    List<String> output = response.map((e) => e.toString()).toList();
    return output;
  }

  /// Check Store token name
  Future<String?> checkScoreTokenName(String scoreAddress) async {
    final String? storeTokenName =
        await _channel.invokeMethod("getScoreTokenName", {
      "score_address": scoreAddress,
    });

    return storeTokenName;
  }

  /// Check Symbol token
  Future<String?> checkScoreTokenSymbol(String scoreAddress) async {
    final String? symbolToken = await _channel.invokeMethod("getTokenSymbol", {
      "score_address": scoreAddress,
    });

    return symbolToken;
  }

  /// Get transaction result
  Future<Map<String, dynamic>?> getTransactionResult(String txHash) async {
    final tResult = await _channel.invokeMethod("getTransactionResult", {
      "tx_hash": txHash,
    });

    return Map<String, dynamic>.from(tResult);
  }

  /// Get block by height
  Future<Map<String, String>?> getBlockByHeight(String blockHeight) async {
    final block = await _channel.invokeMethod("getBlockByHeight", {
      "block_height": blockHeight,
    });

    return Map.from(block);
  }

  /// Get block by transaction hash
  Future<Map<String, String>?> getBlockByHash(String blockHash) async {
    final block = await _channel.invokeMethod("getBlockByHash", {
      "block_hash": blockHash,
    });

    return Map.from(block);
  }

  /// Get block's total supply
  Future<String?> getTotalSupply() async {
    final totalSupply = await _channel.invokeMethod("getTotalSupply");

    return totalSupply;
  }

  /// Get the SCORE apis
  Future<List<Map<String?, String?>>?> getScoreAPIs(String scoreAddress) async {
    final scoreAPIs = await _channel.invokeMethod("getScoreAPIs", {
      "score_address": scoreAddress,
    });

    List<Map<String?, String?>> data = [];
    for (int i = 0; i < scoreAPIs.length; i++) {
      data.add(Map.from(scoreAPIs[i]));
    }

    return data;
  }

  /// Get wallet by private key
  Future<Map<String, String>?> getWalletByPrivateKey(String privateKey) async {
    final wallet = await _channel.invokeMethod("getWalletByPrivateKey", {
      "private_key": privateKey,
    });

    return Map.from(wallet);
  }
}
